@@ -170,5 +170,5 @@ class RoborockSocketListener:
             await self.disconnect()
             raise RoborockTimeout(f"Timeout after {self.timeout} seconds waiting for response") from None
         except BrokenPipeError as e:
-            _LOGGER.exception(e)
             await self.disconnect()
+            raise RoborockException(e) from e
